from AbstractTsDbMixin import AbstractTsDbMixin
from loguru import logger
import iso8601
import datetime
from io import StringIO

class EGM1mdTsDbMixin(AbstractTsDbMixin):
    def write(self, experiment, dataObject):
        logger.info(f"Db write: {experiment} {dataObject}")
        try:
            ts = iso8601.parse_date(dataObject.payload["ts"])
            dt = datetime.timedelta(microseconds=dataObject.payload["dt"])
            values = dataObject.payload["v"]
            logger.info(f"ts: {ts}")
            logger.info(f"dt: {dt}")
            logger.info(f"v: {values}") 
        
            stringBuffer = ''
            for rv in values:
                statusOctet = (rv & 0xff000000) >> 24
                unsignedValue = rv & 0x00ffffff
                valid = (statusOctet & 0x04) >> 2
                sign = (statusOctet & 0x02) >> 1
                signedValue = unsignedValue if not sign else unsignedValue * -1
                
                logger.info(f"store: ts: {ts}, rv: {rv:08x}, status: {statusOctet:02x}, valid: {valid}, sign: {sign}, unsigned: {unsignedValue}, signed: {signedValue}")

                stringBuffer += f"{ts}\t{experiment}\t{rv}\t{valid}\t{signedValue}\n"
                ts += dt

            stringIO = StringIO(stringBuffer)

            logger.info("database copy started")
            with self.connection:
                with self.connection.cursor() as cursor:
                    cursor.copy_from(stringIO, 'egm1md', columns=('time', 'experiment', 'rawvalue', 'valid', 'value'))
            logger.info("database copy done")


        except iso8601.ParseError as e:
            logger.error(f"unable to parse timestamp, item dropped: {f}")
        except KeyError as e:
            logger.error(f"missing attribute in payload, item dropped: {e}")
